﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using EosClocks;

namespace EOSClockDemo
{
    public partial class MainForm : Form
    {
        private Clock _clock;
        private RestaurantClock _restaurantClock;
        private bool _continueGetRecords;
        private bool _continuePcChecking;
        private bool _continueReservePcChecking;

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            PrepareSerialPorts();
        }

        private void PrepareSerialPorts()
        {
            serialPortsCombobox.Items.Clear();
            for (var i = 1; i < 256; i++)
                serialPortsCombobox.Items.Add("COM" + i);
            serialPortsCombobox.SelectedIndex = 0;
            baudRateCombobox.SelectedIndex = 3;
            parityCombobox.Items.AddRange(Enum.GetNames(typeof(System.IO.Ports.Parity)));
            parityCombobox.SelectedIndex = parityCombobox.Items.Count > 0 ? 0 : -1;
            protocolTypeCombobox.SelectedIndex = 0;
        }

        private void btnCreateClock_Click(object sender, EventArgs e)
        {
            Connection connection;
            if (SerialRadioButton.Checked)
            {
                connection = ConnectionFactory.CreateSerialConnection(serialPortsCombobox.SelectedIndex + 1,
                    Convert.ToInt32(baudRateCombobox.Items[baudRateCombobox.SelectedIndex].ToString()),
                    (System.IO.Ports.Parity)(parityCombobox.SelectedIndex),
                    Convert.ToInt32(readTimeoutTextBox.Text),
                    Convert.ToInt32(writeTimeoutTextBox.Text),
                    Convert.ToInt32(waitBeforReadTextBox.Text));
            }
            else
            {
                connection = ConnectionFactory.CreateTCPIPConnection(IPAdressTextBox.Text, 1001,
                    Convert.ToInt32(waitBeforReadTextBox.Text),
                    Convert.ToInt32(readTimeoutTextBox.Text),
                    Convert.ToInt32(writeTimeoutTextBox.Text));
            }

            _clock = new Clock(connection, (ProtocolType)protocolTypeCombobox.SelectedIndex, 
                Convert.ToInt32(addressNUP.Value), ProtocolType.Suprema);

            MessageBox.Show("Clock created successfully");
        }

        private void btnTestConnection_Click(object sender, EventArgs e)
        {
            if (_clock == null)
            {
                MessageBox.Show("Please create clock!");
                return;
            }

            if (_clock.TestConnection())
            {
                var sb = new StringBuilder();
                sb.AppendLine("Clock Model : " + _clock.GetModel());
                sb.AppendLine("Clock Firmware Version : " + _clock.GetFirmwareVersion());
                sb.AppendLine("Clock DateTime : " + _clock.GetDateTime());
                MessageBox.Show(sb.ToString());
            }
            else
            {
                MessageBox.Show("Test Connecton Error", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void SerialRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            SerialPanel.Enabled = SerialRadioButton.Checked;
            TcpIpPanel.Enabled = TcpIpRadioButton.Checked;
        }

        private void btnStartStopGetRecords_Click(object sender, EventArgs e)
        {
            _continueGetRecords = !_continueGetRecords;
            if (_continueGetRecords)
                StartGetRecords();
        }

        private void StartGetRecords()
        {
            btnStartStopGetRecords.Text = "Stop Get Records";
            while (_continueGetRecords)
            {
                if (_clock.IsEmpty())
                {
                    MessageBox.Show("Clock is Empty");
                    break;
                }
                try
                {
                    var record = (ClockRecord)_clock.GetRecord();
                    recordsListBox.Items.Add(record.ToString());
                    _clock.NextRecord();
                }
                catch (Exception ex)
                {
                    if ((ex is EosClocks.InvalidRecordException) || (ex is EosClocks.InvalidDataInRecordException))
                    {
                        // you can find bad record data in ex.Data["RecordRawData"]
                        MessageBox.Show("Bad Record :" + Environment.NewLine + ex.Data["RecordRawData"]);
                    }
                    else 
                        MessageBox.Show(ex.ToString());
                }

                Application.DoEvents();
            }
            _continueGetRecords = false;
            MessageBox.Show("Operation complete successfully");
            btnStartStopGetRecords.Text = "Start Get Records";
        }

        private void btnStartPCChecking_Click(object sender, EventArgs e)
        {
            _continuePcChecking = !_continuePcChecking;
            if (_continuePcChecking)
                StartPCChecking();
        }

        /// <summary>
        /// use this method to response device request for user permission on this device
        /// NOTE 1: for using device with this method device must be online and PCChecking of device must be active and set to Inhibit recording
        /// NOTE 2: for use PCChecking it better to use RS-485 Protocol because in RS-232 mode message1, message2 ignored
        /// </summary>
        private void StartPCChecking()
        {
            btnStartPCChecking.Text = "Stop PC Checking";
            while (_continuePcChecking)
            {
                EosClocks.PCCheckRecord[] pcCheckResults = null;
                try
                {
                    pcCheckResults = _clock.GetPCCheck();
                }
                catch (Exception ex)
                {
                    // log Exception
                }
                if (pcCheckResults != null)
                    if (pcCheckResults.Length > 0)
                    {
                        EosClocks.PCCheckRecord pcRec = pcCheckResults[pcCheckResults.Length - 1];
                        var hasError = true;
                        for (var i = 0; i < _clock.RetryCount; i++)
                            if (_clock.SetPCCheck(rbOK.Checked, txtboxMsg1.Text, txtboxMsg2.Text))
                            {
                                if (rbOK.Checked)
                                {
                                    //at this point you have to choice, 1st simply call NextRecord to remove record from clock because 
                                    //you already get it via GetPCCheck record, or leave it in device an in some idle time call GetRecord to read Records of device
                                    _clock.NextRecord();
                                }
                                hasError = false;
                                break;
                            }
                        if (hasError)
                        {
                            MessageBox.Show("Error in SetPCCheck ", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                System.Threading.Thread.Sleep(100);
                Application.DoEvents();
            }
            _continuePcChecking = false;
            btnStartPCChecking.Text = "Start PC Checking";
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            _continueGetRecords = false;
            _continuePcChecking = false;
            _continueReservePcChecking = false;
        }

        private void btnPreviewRecords_Click(object sender, EventArgs e)
        {
            if (_clock.IsEmpty())
            {
                MessageBox.Show("Clock is Empty");
                return;
            }
            recordsListBox.Items.Clear();
            Application.DoEvents();            

            var clockModel = _clock.GetModel();
            var isStSharp = clockModel.IndexOf("ST-SH") >= 0;

            var readPointer = _clock.GetReadPointer();
            var writePointer = _clock.GetWritePointer();
            int count;
            if (readPointer < writePointer)
                count = writePointer - readPointer;
            else
            {
                if (isStSharp)
                    count = (253579 - readPointer) + writePointer;
                else
                    count = (19500 - readPointer) + writePointer;
            }
            List<string> badRecords;
            var records = _clock.Dump(readPointer, count, out badRecords);
            foreach (var clockRecord in records)
                recordsListBox.Items.Add(clockRecord.ToString());
            MessageBox.Show("OK");
        }

        private void btnCreateRestaurantClock_Click(object sender, EventArgs e)
        {
            Connection connection;
            if (SerialRadioButton.Checked)
            {
                connection = ConnectionFactory.CreateSerialConnection(serialPortsCombobox.SelectedIndex + 1,
                    Convert.ToInt32(baudRateCombobox.Items[baudRateCombobox.SelectedIndex].ToString()),
                    (System.IO.Ports.Parity)(parityCombobox.SelectedIndex),
                    Convert.ToInt32(readTimeoutTextBox.Text),
                    Convert.ToInt32(writeTimeoutTextBox.Text),
                    Convert.ToInt32(waitBeforReadTextBox.Text));
            }
            else
            {
                connection = ConnectionFactory.CreateTCPIPConnection(IPAdressTextBox.Text, 1001,
                    Convert.ToInt32(waitBeforReadTextBox.Text),
                    Convert.ToInt32(readTimeoutTextBox.Text),
                    Convert.ToInt32(writeTimeoutTextBox.Text));
            }

            _restaurantClock = new RestaurantClock(connection, (ProtocolType)protocolTypeCombobox.SelectedIndex,
                Convert.ToInt32(addressNUP.Value), ProtocolType.Suprema);

            MessageBox.Show("Restaurant Clock created successfully");

        }

        private void btnRestaurantClockTestConnection_Click(object sender, EventArgs e)
        {
            if (_restaurantClock == null)
            {
                MessageBox.Show("Please create Restaurant Clock!");
                return;
            }

            if (_restaurantClock.TestConnection())
            {
                var sb = new StringBuilder();
                sb.AppendLine("Clock Model : " + _restaurantClock.GetModel());
                sb.AppendLine("Clock Firmware Version : " + _restaurantClock.GetFirmwareVersion());
                sb.AppendLine("Clock DateTime : " + _restaurantClock.GetDateTime());
                MessageBox.Show(sb.ToString());
            }
            else
            {
                MessageBox.Show("Test Connecton Error", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnSendValidList_Click(object sender, EventArgs e)
        {
            if (_restaurantClock == null)
            {
                MessageBox.Show("Please create Restaurant Clock!");
                return;
            }

            var validList = new List<RestaurantValidList>();
            validList.Add(new RestaurantValidList("4451742", true, true, true, false, true));
            validList.Add(new RestaurantValidList("102030", false, false, false, false, false));
            validList.Add(new RestaurantValidList("123456", true, false, false, true, false));
            validList.Add(new RestaurantValidList("18955674"));

            if (_restaurantClock.SendValidCards(validList))
                MessageBox.Show("SendValidCards completed successfully");
        }
        /// <summary>
        /// for defining meals of each day of week you have to use SendDayMealsData method for each day
        /// </summary>
        private void btnSendDayMealsData_Click(object sender, EventArgs e)
        {
            if (_restaurantClock == null)
            {
                MessageBox.Show("Please create Restaurant Clock!");
                return;
            }

            var restaurantDayMealsDataList = new List<RestaurantDayMealsData>(7)
                                             {
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Saturday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Sunday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Monday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Tuesday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Wednesday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Thursday),
                                                 new RestaurantDayMealsData("علم و صنعت", DayOfWeek.Friday)
                                             };
            progressBar1.Step = 1;
            progressBar1.Minimum = 1;
            progressBar1.Maximum = restaurantDayMealsDataList.Count;
            progressBar1.Value = 1;
            lblMsg.Text = string.Empty;
            progressBar1.Visible = true;
            lblMsg.Visible = true;
            Application.DoEvents();
            try
            {
                foreach (var dayMealsData in restaurantDayMealsDataList)
                {
                    lblMsg.Text = "Sending " + dayMealsData.DayOfWeek;
                    Application.DoEvents();

                    for (var i = 0; i < dayMealsData.MealsData.Length; i++)
                    {
                        dayMealsData.MealsData[i].MealName = "وعده " + (i + 1);
                        switch (i)
                        {
                            case 0:
                                dayMealsData.MealsData[i].StartTime = "05:00";
                                dayMealsData.MealsData[i].EndTime = "07:00";
                                dayMealsData.MealsData[i].Food1Name = "آش رشته";
                                dayMealsData.MealsData[i].Food2Name = "حلیم";
                                dayMealsData.MealsData[i].Food3Name = "نان وپنیر";
                                dayMealsData.MealsData[i].Dessert1Name = "چای";
                                dayMealsData.MealsData[i].Dessert2Name = "شیرگرم";
                                break;
                            case 1:
                                dayMealsData.MealsData[i].StartTime = "08:00";
                                dayMealsData.MealsData[i].EndTime = "09:30";
                                dayMealsData.MealsData[i].Food1Name = "نیمرو";
                                dayMealsData.MealsData[i].Food2Name = "سوسیس";
                                dayMealsData.MealsData[i].Food3Name = "نان و عسل";
                                dayMealsData.MealsData[i].Dessert1Name = "قهوه";
                                dayMealsData.MealsData[i].Dessert2Name = "نسکافه";
                                break;
                            case 2:
                                dayMealsData.MealsData[i].StartTime = "11:30";
                                dayMealsData.MealsData[i].EndTime = "13:30";
                                dayMealsData.MealsData[i].Food1Name = "چلوکباب";
                                dayMealsData.MealsData[i].Food2Name = "پیتزا";
                                dayMealsData.MealsData[i].Food3Name = "جوجه کباب";
                                dayMealsData.MealsData[i].Dessert1Name = "نوشابه";
                                dayMealsData.MealsData[i].Dessert2Name = "سالاد";

                                break;
                            case 3: // for disable each meal use this approach 
                                dayMealsData.MealsData[i].StartTime = string.Empty;
                                dayMealsData.MealsData[i].EndTime = string.Empty;
                                dayMealsData.MealsData[i].Food1Name = string.Empty;
                                dayMealsData.MealsData[i].Food2Name = string.Empty;
                                dayMealsData.MealsData[i].Food3Name = string.Empty;
                                dayMealsData.MealsData[i].Dessert1Name = string.Empty;
                                dayMealsData.MealsData[i].Dessert2Name = string.Empty;
                                break;
                            case 4:
                                dayMealsData.MealsData[i].StartTime = "22:00";
                                dayMealsData.MealsData[i].EndTime = "23:30";
                                dayMealsData.MealsData[i].Food1Name = "قرمه سبزی";
                                dayMealsData.MealsData[i].Food2Name = "خوراک برگ";
                                dayMealsData.MealsData[i].Food3Name = "فسنجان";
                                dayMealsData.MealsData[i].Dessert1Name = "دلستر";
                                dayMealsData.MealsData[i].Dessert2Name = "دوغ";
                                break;
                        }
                    }

                    if (!_restaurantClock.SendDayMealsData(dayMealsData))
                        MessageBox.Show("SendDayMealsData Error : " + dayMealsData.DayOfWeek);

                    progressBar1.PerformStep();
                    Application.DoEvents();
                }
                MessageBox.Show("SendDayMealsData completed successfully");
            }
            finally
            {
                progressBar1.Visible = false;
                lblMsg.Visible = false;
            }
        }
        /// <summary>
        /// in reserve mode you must prepare list of people and week reserve status of 
        /// them (depend on Week reserve data that you send to device via SendDayMealsData) 
        /// and send it to device in beginning of each week, then device can work in standalone state without need any online PC connection
        /// NOTE 1: when you send week reserve list, this new list overwrite old reserve list of device
        /// NOTE 2: for use device in reserve mode you have to set device to work in reserve mode via device setup->3->6->1
        /// NOTE 3: when device work in reserve mode, for each day only Meal 0 and Meal 1 of SendDayMealsData is valid and Meal 2 until 5 ignored 
        /// </summary>
        private void btnSendWeekReserveList_Click(object sender, EventArgs e)
        {
            if (_restaurantClock == null)
            {
                MessageBox.Show("Please create Restaurant Clock!");
                return;
            }

            var weekReserveList = new List<RestaurantPersonWeekReserveData>();

            var personWeekReserveData = new RestaurantPersonWeekReserveData("4451742", true, true, false, false, false, 2);
            foreach (DayOfWeek day in Enum.GetValues(typeof(DayOfWeek)))
            {
                // meal 1
                personWeekReserveData[day].Meal1.AllowReserve = true;
                personWeekReserveData[day].Meal1.LimitCardRepeat = false;
                personWeekReserveData[day].Meal1.ReservedFood = ReservedFood.Food1;
                personWeekReserveData[day].Meal1.Dessert1Reserved = true;
                personWeekReserveData[day].Meal1.Dessert2Reserved = false;

                // meal 2
                personWeekReserveData[day].Meal2.AllowReserve = true;
                personWeekReserveData[day].Meal2.LimitCardRepeat = true;
                personWeekReserveData[day].Meal2.ReservedFood = ReservedFood.Food2;
                personWeekReserveData[day].Meal2.Dessert1Reserved = true;
                personWeekReserveData[day].Meal2.Dessert2Reserved = true;
            }

            weekReserveList.Add(personWeekReserveData);

            if (!_restaurantClock.SendWeekReserveList(weekReserveList))
                MessageBox.Show("SendWeekReserveList Error : ");

            MessageBox.Show("SendWeekReserveList completed successfully");

        }

        private void btnUsePcChecking_Click(object sender, EventArgs e)
        {
            if (_restaurantClock == null)
            {
                MessageBox.Show("Please create Restaurant Clock!");
                return;
            }
            
            _continueReservePcChecking = !_continueReservePcChecking;
            if (_continueReservePcChecking)
                StartReservePCChecking();
        }
        /// <summary>
        /// In Reserve PC Checking device must be online with one PC connection 
        /// in this mode, there is no need to use SendWeekReserveList method because device get person reserve data via PC online connection
        /// </summary>
        private void StartReservePCChecking()
        {
            btnUsePcChecking.Text = "Stop Reserve PC Checking";

            while (_continueReservePcChecking)
            {
                EosClocks.PCCheckRecord[] pcCheckResults = null;
                try
                {
                   pcCheckResults = _restaurantClock.GetPCCheck();
                }
                catch (Exception ex)
                {
                    // log Exception
                }
                if (pcCheckResults != null)
                    if (pcCheckResults.Length > 0)
                    {
                        EosClocks.PCCheckRecord pcRec = pcCheckResults[pcCheckResults.Length - 1];
                        if (pcRec.ID == 4451742)
                        {
                            var hasError = true;
                            for(var i=0; i<_restaurantClock.RetryCount; i++)
                                if (_restaurantClock.SetReservePcCheck(true, "علی احمدی", "ناهار", new ReserveData(false, ReservedFood.Food1, true, false)))
                                {
                                    //at this point you have to choice, 1st simply call NextRecord to remove record from clock because 
                                    //you already get it via GetPCCheck record, or leave it in device an in some idle time call GetRecord to read Records of device
                                    _restaurantClock.NextRecord();

                                    hasError = false;
                                    break;
                                }
                            if (hasError)
                            {
                                MessageBox.Show("Error in SetReservePcCheck ", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        else
                            _restaurantClock.SetReservePcCheck(false, "خطا", "کارت غیر مجاز", null);
                    }
                System.Threading.Thread.Sleep(100);
                Application.DoEvents();
            }
            _continuePcChecking = false;
            btnUsePcChecking.Text = "Start Reserve PC Checking";
        }

        private void btnShowActiveNetworkModules_Click(object sender, EventArgs e)
        {
            FrmActiveNetworkModules frmActiveNetworkModules = new FrmActiveNetworkModules();
            if (frmActiveNetworkModules.ShowDialog() == DialogResult.OK)
                IPAdressTextBox.Text = frmActiveNetworkModules.GetSelectedIPAddress();
        }

    }

}
